% Repairing tool for skeleton images

function fig = SkeletonGUI(Istack)

% read projected image stack
%Istack=imread3(filename);

s=size(Istack);

% Create new figure        
fig = figure;

fig1=[];

% initial configuration
frame=1;

frames=s(3);

width=s(2); height=s(1);

PaintMode = 0; % 1=enable eraser; 2=paint pixels
eraseCircle=10;
mpos=[0,0];
drag=0; % indicating mouse drag

eraserSize=1;

% editing mode 
% 1=markers, 2=edit skeleton, 
MarkerMode=1;

% 1=original images, 2=skeletons, 3=skeleton + rgb overlay
ViewMode=1;

% indicates whether to show the overlay image
RGBoverlay=false;

contrast1=200;

marking=0;

ratio=0.5;

skelthreshold=2;

ZoomMode=1; % normal
ZoomCord=[ 3*height/8,3*width/8,5*height/8,5*width/8];


% cell markers and tracks data
markermap=zeros(s(1),s(2),frames,'uint8');
modifiedmarkers=zeros(s(1),s(2),frames,'uint8');
trackmap=zeros(s(1),s(2),frames,'uint16');
Skeletons=zeros(s(1),s(2),frames,'uint8');


% tracking data
tracklength=[];
Ptracks=[];
trackstarts=[];
trackstartX=[]; trackstartY=[]; 
RGBoverlay=[];



% ------- Setting Up GUI -------------

set(fig,...
    'Name', 'Sketelon Tool',... 
    'Color', [0.314 0.314 0.314],...
    'Units', 'normalized',...
    'MenuBar','none',...
    'Position', [0 0 0.8 0.8]);

    % create long slider to control image time frame
    slider1 = uicontrol( fig ...
        ,'style'    ,'slider'   ...
        ,'units'    ,'normalized' ...
        ,'position' ,[0.15 0.05 0.80 0.03] ...
        );

    set(slider1,'min', 1);
    set(slider1,'max', frames);
    set(slider1,'Value', frame);

    addlistener(slider1,'ContinuousValueChange',@sliderActionEventTime);
    
    FrameNumLabel = uicontrol(fig ...
        ,'style'    ,'text' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'position' ,[0.05 0.05 0.09 0.03] ...
        ,'string'   ,sprintf('Frame %d/%d',frame,frames) ...
        );
    
    % contrast slider
    
    slider3 = uicontrol( fig ...
        ,'style'    ,'slider'   ...
        ,'units'    ,'normalized' ...
        ,'position' ,[0.05 0.85 0.07 0.02] ...
        );

    set(slider3,'min', 0);
    set(slider3,'max', 400);
    set(slider3,'Value', 200);
    addlistener(slider3,'ContinuousValueChange',@sliderActionEventContrast);
    
    
    ContrastLabel = uicontrol(fig ...
        ,'style'    ,'text' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 11 ...
        ,'position' ,[0.05 0.87 0.07, 0.02] ...
        ,'string'   , sprintf('Contrast %1.2f',contrast1/100) ...
        );
    
       
    slider4 = uicontrol( fig ...
        ,'style'    ,'slider'   ...
        ,'units'    ,'normalized' ...
        ,'position' ,[0.4 0.9 0.07 0.02] ...
        );

    set(slider4,'min', 0);
    set(slider4,'max', 100);
    set(slider4,'Value',50);
    addlistener(slider4,'ContinuousValueChange',@sliderActionEventRatio);
    
    
    RatioLabel = uicontrol(fig ...
        ,'style'    ,'text' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 11 ...
        ,'position' ,[0.4 0.92 0.07 0.02] ...
        ,'string'   , sprintf('Ratio %1.2f',ratio) ...
        );
    
    
    set (gcf, 'WindowButtonMotionFcn', @MouseMoveEvent);
    
    set(fig,'KeyPressFcn',@KeyPressEvent);
    
    
    % ------  buttons --------------
    
    SuggestButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Find Cells'...
        ,'Position', [0.05 0.9 0.1 0.04]...
        ,'Callback', @SuggestMarkers ... 
        );
    
    
    TrackBackwardButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', '<<'...
        ,'Position', [0.15 0.9 0.05 0.04]...
        ,'Callback', @TrackMarkersBackward ... 
        );
    
    TrackForwardButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', '>>'...
        ,'Position', [0.2 0.9 0.05 0.04]...
        ,'Callback', @TrackMarkersForward ... 
        );
    
    
    ClearButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Clear Frame'...
        ,'Position', [0.25 0.95 0.1 0.04]...
        ,'Callback', @ClearMarkers ... 
        );
    
    ClearAllButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Clear All'...
        ,'Position', [0.25 0.9 0.1 0.04]...
        ,'Callback', @ClearAllMarkers ... 
        );
    
   
    
    CreateButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Create Tracks'...
        ,'Position', [0.6 0.9 0.1 0.04]...
        ,'Callback', @CreateTracks ... 
        );
    
     PlayButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 12 ...
        ,'String', '>'...
        ,'Position', [0.01 0.05 0.03 0.03]...
        ,'Callback', @PlaySequence ... 
        );
    
       
%    ViewMarkersMenu = uicontrol('Style', 'popup',...
%            'String', {'View markers','View tracks'}...
%            ,'units'    ,'normalized' ...
%            ,'FontSize', 14 ...
%            ,'Position', [0.37 0.94 0.15 0.05]...
%            ,'Callback', @SetViewMarkersMode...
%            );       
       
    ViewImageMenu = uicontrol('Style', 'popup',...
           'String', {'Image Mode','Skeleton Mode', 'Overlay'}...
           ,'units'    ,'normalized' ...
           ,'FontSize', 14 ...
           ,'Position', [0.37 0.94 0.15 0.05]...
           ,'Callback', @SetImageMode...
           );    
    
     LoadSkeletonButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Load Skeleton'...
        ,'Position', [0.75 0.9 0.1 0.04]...
        ,'Callback', @LoadSkeletons ... 
        ); 
       
      
     LoadMarkersButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Load Markers'...
        ,'Position', [0.75 0.95 0.1 0.04]...
        ,'Callback', @LoadTracks ... 
        );
    
    FindAllCellsButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Find ALL'...
        ,'Position', [0.05 0.95 0.1 0.04]...
        ,'Callback', @FindAllCells ... 
        );
    
    SaveMarkersButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Save Markers'...
        ,'Position', [0.85 0.95 0.1 0.04]...
        ,'Callback', @SaveTracks ... 
        );
    
     ExportSkeletonButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Save Skeleton'...
        ,'Position', [0.85 0.9 0.1 0.04]...
        ,'Callback', @ExportSkeleton ... 
        );
   
    FeedbackLabel = uicontrol(fig ...
        ,'style'    ,'text' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'position' ,[0 0 0.15, 0.03] ...
        ,'string'   ,'Ready' ...
        );

Update();




function Update()
    
        set(FeedbackLabel,'string','Ready');
        
       
        figure(fig);
       
        set(slider1,'Value', frame);
        
        % ---------- Show Stack -----------------
        
        img=imshow(createOverlayImage);
        
         % show eraser marker
        if (PaintMode == 1)
        
            hold on,
            
              fig4=plot(round(mpos(1)),round(mpos(2)),'ro','MarkerSize', 6*eraserSize); 
              hold off
              set(fig4,'ButtonDownFcn',@MouseButtonDown);  
            
            
        end
        
         % draw pixel marker
         if (PaintMode == 2)
        
            hold on,
            
            fig4=plot(mpos(1),mpos(2),'yo','MarkerSize', eraseCircle); 
            hold off
            set(fig4,'ButtonDownFcn',@MouseButtonDown);
            
        end
        
       
        % no zoom
        if (ZoomMode==1)
        
        for n=1:3,
        xp(1)=n*width/4; xp(2)=n*width/4;
        yp(1)=0;yp(2)=height;
        line(xp,yp,'LineStyle','--');
        
        yp(1)=n*height/4; yp(2)=n*height/4;
        xp(1)=0;xp(2)=width;
        line(xp,yp,'LineStyle','--');
        end
        
        
        % unallocated markers
        [vy,vx]=find(markermap(:,:,frame)>=253);
        hold on
        fig1=plot(vx,vy,'r*','MarkerSize', 5); 
        hold off
        
         % tracked markers
        [vy,vx]=find(trackmap(:,:,frame)>0);
        hold on
        fig2=plot(vx,vy,'g*','MarkerSize', 5); 
        hold off
        
        % dividing cells
        [vy,vx]=find(markermap(:,:,frame)==253);
        hold on
        fig3=plot(vx,vy,'y*','MarkerSize', 10); 
        hold off 
        
        % zoom
        else
              
            
        [vy,vx]=find(markermap(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame)>=253);
        hold on
        fig1=plot(vx,vy,'r*','MarkerSize', 5); 
        hold off 
        
        [vy,vx]=find(trackmap(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame)>0);
        hold on
        fig2=plot(vx,vy,'g*','MarkerSize', 5); 
        hold off
        
        [vy,vx]=find(markermap(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame)==253);
        hold on
        fig3=plot(vx,vy,'y*','MarkerSize', 15); 
        hold off 
         
        end
        
        set(fig1,'ButtonDownFcn',@MouseButtonDown);
        set(fig2,'ButtonDownFcn',@MouseButtonDown);
        set(fig3,'ButtonDownFcn',@MouseButtonDown);
        set(img,'ButtonDownFcn',@MouseButtonDown);
        
        set(gcf, 'WindowButtonUpFcn',@MouseButtonUp);
        
        set(FrameNumLabel,'string',sprintf('Frame %d/%d',frame,frames));
        
       
        
end

% --- Event Handlers -----------------------


function sliderActionEventTime(src,evt)
        frame = round(get(src,'Value'));        
        Update();     
end



function sliderActionEventContrast(src,evt)
        contrast1 = round(get(src,'Value'));
        set(ContrastLabel,'string',sprintf('Contrast %1.2f',contrast1/100));
        Update();
end


function sliderActionEventRatio(src,evt)
        ratio = round(get(src,'Value'))/100;
        set(RatioLabel,'string',sprintf('Ratio %1.2f',ratio));
        Update();
end


function SuggestMarkers(src,evt)
        
    set(FeedbackLabel,'string',' Finding cells in frame .....');
    drawnow;
   
            
    centroids = FindCellsFromSkeletonFunc(ones(size(Skeletons(:,:,frame))),Skeletons(:,:,frame),skelthreshold);
         
    % delete current markers
    markermap(:,:,frame) = zeros(s(1),s(2),'uint8');
         
         for n=1:length(centroids)
        
         if(centroids(n,1) && centroids(n,2) > 0)
          
          markermap(centroids(n,2),centroids(n,1),frame)=255;
         end
         end
         
         markermap(:,:,frame)=markermap(:,:,frame)+modifiedmarkers(:,:,frame);
         
         [y,x]=find(modifiedmarkers(:,:,frame)==1);
         
         for n=1:length(y)
            markermap(y(n),x(n),frame)=0; 
         end
         
         
         Update();
                 
    
    set(FeedbackLabel,'string','Ready');
    drawnow;
        
end



function FindAllCells(src,evt)
        
    set(FeedbackLabel,'string',' Finding cells .....');
    drawnow;
    
   
       startframe=1;
       stopframe=frames;
       %stopframe=5;
      
%          centroids = FindCellsFunc(Skeletons(:,:,frame),2);
%         
%         % delete current markers
%         markermap(:,:,frame) = zeros(s(1),s(2),'uint8');
%         
%         for n=1:length(centroids)
%         
%          if(centroids(n,1) && centroids(n,2) > 0)
%           
%           markermap(centroids(n,2),centroids(n,1),frame)=255;
%          end
%         end
        
        % adding existing markers or removing markers
        
        
        for t=startframe:stopframe,
            
        set(FeedbackLabel,'string',' Finding cells .....');
        drawnow;
            
        frame=t;
        
        
         centroids = FindCellsFromSkeletonFunc(ones(size(Skeletons(:,:,frame))),Skeletons(:,:,frame),skelthreshold);
         
          % delete current markers
          markermap(:,:,frame) = zeros(s(1),s(2),'uint8');
         
         for n=1:length(centroids)
        
         if(centroids(n,1) && centroids(n,2) > 0)
          
          markermap(centroids(n,2),centroids(n,1),frame)=255;
         end
         end
         
         Update();
         drawnow;
         
        end
        
    
    set(FeedbackLabel,'string','Ready');
    Update();
    
end


function TrackMarkersBackward(src,evt)
        
    set(FeedbackLabel,'string',' Track cells .....');
    drawnow;
  
    if (frame <= frames && frame > 1)
      
     frame=frame-1;
 
    end
    
    set(FeedbackLabel,'string','Ready');
    
    Update();
        
end



function TrackMarkersForward(src,evt)
        
    set(FeedbackLabel,'string',' Track cells .....');
    drawnow;
  
    if (frame < frames && frame >= 1)
     frame=frame+1;
 
    end
    
    set(FeedbackLabel,'string','Ready');
    
    Update();
        
end


function CreateTracks(src,evt)
        
    set(FeedbackLabel,'string',' Creating tracks .....');
    drawnow;
    
    %[trackmap, tracklength]=cellTracking(markermap, 20);
    [trackmap, Ptracks, tracklength, trackstarts, trackstartX, trackstartY ]=cellTracking4( markermap,20);
    
    set(FeedbackLabel,'string','Ready');
    
    
    for n=1:length(trackstarts)
        
       trackmap(trackstartY(n), trackstartX(n), trackstarts(n))=0;
    end
    
    %IdentifyDivisions();
    
    Update();
        
end



function IdentifyDivisions()
    
    for f=1:frames,
    
    Ilabel=bwlabel(Skeletons(:,:,f)<10);
    
     [vy,vx]=find(markermap(:,:,f)>=252);
     
     
     % first iteration
     for n=1:length(vy)
       
         % mark as dividing cells
         if (Ilabel(vy(n),vx(n)) < 0)
             
            markermap(vy(n),vx(n),f)=253;
            
         end
         
         if (Ilabel(vy(n),vx(n)) > 0)
             
             col=Ilabel(vy(n),vx(n));
             Ilabel(Ilabel==col)=-1;   
         end  
     end
     
     
     Ilabel=bwlabel(Skeletons(:,:,f)<10);
     
     % second iteration
     for n=1:length(vy)
         
         % mark as dividing cells
         if (markermap(vy(n),vx(n),f)==253)
            
             col=Ilabel(vy(n),vx(n));
             Ilabel(Ilabel==col)=-1; 
             
         end
         
     end
     % third iteration
      for n=1:length(vy)
         
          if (Ilabel(vy(n),vx(n)) < 0)
             
            markermap(vy(n),vx(n),f)=253;
            
         end
         
             
     end
     
    end
           
end



function PlaySequence(src,evt)
    
    if (frame < frames)
       frame=frame+1;
       Update();
        
    end
        
end



function ExportSequence2(src,evt)
    
    filename=uiputfile('*.tif','Export Image Sequence');
    if (filename==0)
     disp('no file selected');
    else
        
        set(FeedbackLabel,'string',' Exporting sequence .....');
        drawnow;
        
        for f=1:frames,
            
         Irgb(:,:,1)=Skeletons(:,:,f); %R
         Irgb(:,:,2)=Skeletons(:,:,f); %G
         Irgb(:,:,3)=Skeletons(:,:,f); %B
     
        % unallocated markers (red)
        [vy,vx]=find(markermap(:,:,f) > 253);
        
        for n=1:length(vx),
            
           if (trackmap(vy(n),vx(n)) < 1) 
           Irgb(vy(n),vx(n),1)=255;
           Irgb(vy(n)-1,vx(n),1)=255;
           Irgb(vy(n)+1,vx(n),1)=255;
           Irgb(vy(n),vx(n)-1,1)=255;
           Irgb(vy(n),vx(n)+1,1)=255; 
           end
        end
    
         % tracked markers (green)
        [vy,vx]=find(trackmap(:,:,f) > 0);
        
        for n=1:length(vx),
           Irgb(vy(n),vx(n),2)=255;
           Irgb(vy(n)-1,vx(n),2)=255;
           Irgb(vy(n)+1,vx(n),2)=255;
           Irgb(vy(n),vx(n)-1,2)=255;
           Irgb(vy(n),vx(n)+1,2)=255;  
           
        end
        
 
        % dividing cells (yellow)
        [vy,vx]=find(markermap(:,:,f)==253);
        for n=1:length(vx),
           Irgb(vy(n),vx(n),3)=255;
           Irgb(vy(n)-1,vx(n),3)=255;
           Irgb(vy(n)+1,vx(n),3)=255;
           Irgb(vy(n),vx(n)-1,3)=255;
           Irgb(vy(n),vx(n)+1,3)=255;
           Irgb(vy(n),vx(n),2)=255;
           Irgb(vy(n)-1,vx(n),2)=255;
           Irgb(vy(n)+1,vx(n),2)=255;
           Irgb(vy(n),vx(n)-1,2)=255;
           Irgb(vy(n),vx(n)+1,2)=255;   
        end
        
        % saving images
        
        if (f==1) imwrite( Irgb,filename,'tif','Compression','none');
        else
          imwrite(Irgb,filename,'tif','WriteMode','append','Compression','none');   
        end
        
        end 
        
        
    end
     
     set(FeedbackLabel,'string',' Ready .....');
     drawnow;
end


function ClearMarkers(src,evt)
        
  % erase all markers in frame
  markermap(:,:,frame) = zeros(s(1),s(2),'uint8');
  modifiedmarkers(:,:,frame) = zeros(s(1),s(2),'uint8');
  trackmap(:,:,frame) = zeros(s(1),s(2),'uint16');
  Update();
        
end


function ClearAllMarkers(src,evt)
        
  % erase all markers in stack
  markermap=zeros(s(1),s(2),frames,'uint8');
  modifiedmarkers=zeros(s(1),s(2),frames,'uint8');
  trackmap=zeros(s(1),s(2),frames,'uint16');
  Update();
        
end





function SetImageMode(src,evt)
        
        ViewMode=get(src,'Value');
        Update();
        
end

function SetViewMarkersMode(src,evt)
        
        MarkerMode=get(src,'Value');
        Update();
        
end



function LoadSkeletons(src,evt)
        
    filename=uigetfile('*.tif','Load Skeletons');
    if (filename==0)
     disp('no file selected');
    else
      set(FeedbackLabel,'string','Loading .....');
      drawnow;
      Skeletons=imread3(filename);
      disp(filename);  
      skelthreshold=mean(Skeletons(:));
    end
    ViewMode=2;
    set(ViewImageMenu,'Value', 2);
    Update();  
end


function ExportSkeleton(src,evt)
    
    filename=uiputfile('*.tif','Save Skeleton');
    if (filename==0)
     disp('no file selected');
    else
      imwrite3(uint8(Skeletons),filename);
       disp(filename);
    end 
    
    
end


function LoadRGBImages(src,evt)
        
    filename=uigetfile('*.tif','Load Skeletons');
    if (filename==0)
     disp('no file selected');
    else
      set(FeedbackLabel,'string','Loading .....');
      drawnow;
      RGBstack=imread3(filename);
      disp(filename);
      RGBoverlay=true;
    end
    ViewMode=3;
    set(ViewImageMenu,'Value', 3);
    Update();  
end



function img=createOverlayImage()
    
 img=[];

 % rgb overlay
 if (ViewMode==3)
    if (ZoomMode > 1)      
        I1=Skeletons(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);
        I2=Istack(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);
    else
        I1=Skeletons(:,:,frame); 
        I2=Istack(:,:,frame);
    end 
    
    img1(:,:,1)=zeros(size(I1));
    img1(:,:,2)=double(I2)*contrast1/100;
    img1(:,:,3)=zeros(size(I1));
    
    img2(:,:,1)=1.2*double(I1)*contrast1/200;
    img2(:,:,2)=1.2*double(I1)*contrast1/200;
    img2(:,:,3)=1.2*double(I1)*contrast1/200;
    
    %img=uint8(ratio*double(img1)+(double(img2)));
   
    img=uint8(ratio*double(img1)+((1-ratio)*double(img2)));
    
 else
    % no overlay
     if (ZoomMode > 1)
            if (ViewMode == 2)
             img=Skeletons(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);   
            else
             img=Istack(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);
            end
        else
          if (ViewMode == 2)
           img=Skeletons(:,:,frame);  
          else
           img=Istack(:,:,frame);
          end
        
     end
     
     
        
 end
     
end



% ---- handling mouse buttons ----

function MouseButtonDown(src,evt)
    
       pt = get(gca,'Currentpoint');
       
       mouseclick  = get(gcf,'SelectionType');
       
       if (PaintMode >= 1) 
           drag=1;
           marking=0;
           return;  
       end
       
       if strcmp(mouseclick ,'normal')
       
       crd = round([pt(1,1), pt(1,2)]);
       
       
       
       if (ZoomMode ==1)
           crd = round([pt(1,1), pt(1,2)]);
       else
           crd = round([pt(1,1)+ZoomCord(2), pt(1,2)+ZoomCord(1)]);
       end
       
  
       % check bounds
       if (crd(1) < 1) crd(1)=1; end
       if (crd(1) > s(2)) crd(1)=s(2); end
       if (crd(2) < 1) crd(2)=1; end
       if (crd(2) > s(1)) crd(2)=s(1); end
       
       % comment this out
       % markermap(crd(2), crd(1), frame)=255;
       % modifiedmarkers(crd(2), crd(1), frame)=255;
        Update();    
       end
       
       if strcmp(mouseclick ,'alt')
           
          crd = round([pt(1,1), pt(1,2)]);
          
          if (ZoomMode ==1)
           crd = round([pt(1,1), pt(1,2)]);
          else
           crd = round([pt(1,1)+ZoomCord(2), pt(1,2)+ZoomCord(1)]);
          end
          
          % check bounds
          if (crd(1) < 1) crd(1)=1; end
          if (crd(1) > s(2)) crd(1)=s(2); end
          if (crd(2) < 1) crd(2)=1; end
          if (crd(2) > s(1)) crd(2)=s(1); end
          
          removeMarker(crd(2), crd(1), 5);
          Update();
           
       end  
end


function MouseButtonUp(src,evt)
    
       if (PaintMode==1 && eraserSize==1)
           
        pt = get(gca,'Currentpoint');
       
        mouseclick  = get(gcf,'SelectionType');
            
        crd = round([pt(1,1), pt(1,2)]);
              
       if (ZoomMode ==1)
           crd = round([pt(1,1), pt(1,2)]);
       else
           crd = round([pt(1,1)+ZoomCord(2), pt(1,2)+ZoomCord(1)]);
       end
       
       % check bounds
       if (crd(1) < 1) crd(1)=1; end
       if (crd(1) > s(2)) crd(1)=s(2); end
       if (crd(2) < 1) crd(2)=1; end
       if (crd(2) > s(1)) crd(2)=s(1); end
   
         removeMarker(crd(2), crd(1), 5); 
      
       end
       
    drag=0;  marking=0;
end

function MouseMoveEvent (object, eventdata)
    
     pt = get (gca, 'CurrentPoint');
     mpos=[pt(1,1) pt(1,2)];
    

    if (PaintMode >= 1)
        
        % deleting skeleton
        if (PaintMode==1 && drag==1)
           removeSkeleton(round(mpos(1)), round(mpos(2)), eraserSize);
           marking=marking+1;
        end
        
        % drawing skeleton
        if (PaintMode==2 && drag==1)
           paintSkeleton(round(mpos(1)), round(mpos(2)), 1); 
           marking=marking+1;
        end
        
        % dealing with the updating after drawing every so often
        if (marking > 9)
        Update();  
        marking=0; 
        end
        
        if (drag==0) 
            Update();
        end
    end
    

end


function removeMarker(x,y, mindistance)
    
%      if (ZoomMode ==2)
%        x=x+ZoomCord(1);
%        y=y+ZoomCord(2);
%      end
    
   
     [vx,vy]=find(markermap(:,:,frame)>0);
    
     for n=1:length(vx)
         if (hypot(vx(n)-x,vy(n)-y) < mindistance)
           
            markermap(vx(n),vy(n),frame)=0; 
            modifiedmarkers(vx(n),vy(n),frame)=1; 
             
         end    
     end
     
  
     [vx,vy]=find(trackmap(:,:,frame)>0);
    
     for n=1:length(vx)
         if (hypot(vx(n)-x,vy(n)-y) < mindistance)
           
            trackmap(vx(n),vy(n),frame)=0; 
             
         end    
     end
     
end



function paintSkeleton(x,y,size)

    
    if (ZoomMode ==2)
       x=x+ZoomCord(2);
       y=y+ZoomCord(1);
    end
   
     if (x < size) x=size; end
     if (x > s(2)-size) x=s(2)-size; end
     if (y < size) y=size; end
     if (y > s(1)-size) y=s(1)-size; end


     Skeletons(y,x,frame)=150;
%      Skeletons(y-1,x,frame)=150;
%      Skeletons(y+1,x,frame)=150;
%      Skeletons(y,x-1,frame)=150;
%      Skeletons(y,x+1,frame)=150;
%      Skeletons(y-1,x-1,frame)=100;
%      Skeletons(y+1,x+1,frame)=100;
%      Skeletons(y+1,x-1,frame)=100;
%      Skeletons(y-1,x+1,frame)=100;
     
   
end



function removeSkeleton(x,y,size)

    
    if (ZoomMode ==2)
       x=x+ZoomCord(2);
       y=y+ZoomCord(1);
    end
   
     if (x < size) x=size; end
     if (x > s(2)-size) x=s(2)-size; end
     if (y < size) y=size; end
     if (y > s(1)-size) y=s(1)-size; end

     Skeletons(y-size:y+size,x-size:x+size,frame)=0;
   
end



function ZoomCoordinatesPoint()
        
        
        
                    if (mpos(1) <= width/8)
                        mpos(1)=width/8+1;
                    end
                    if (mpos(2) <= height/8)
                        mpos(2)=height/8+1;
                    end
                    
                    if (mpos(1) >= 7*width/8)
                        mpos(1)=7*width/8;
                    end
                    if (mpos(2) >= 7*height/8)
                        mpos(2)=7*height/8;
                    end
                 
                       
                    ZoomCord=[mpos(2)-height/8,mpos(1)-width/8,mpos(2)+height/8,mpos(1)+width/8]; 
                    ZoomCord=round(ZoomCord);
        
end



function ZoomCoordinatesRegion()
        
        
                    
                    if (mpos(1) < width/4)
                        mpos(1)=width/8+1; 
                    elseif(mpos(1) < width/2)
                         mpos(1)=3*width/8+1;   
                    elseif(mpos(1) < 3*width/4)
                         mpos(1)=5*width/8+1;
                    else
                        mpos(1)=7*width/8;
                    end
                    
                    
                    if (mpos(2) < height/4)
                        mpos(2)=height/8+1; 
                    elseif(mpos(2) < height/2)
                         mpos(2)=3*height/8+1;   
                    elseif(mpos(2) < 3*height/4)
                         mpos(2)=5*height/8+1;
                    else
                        mpos(2)=7*height/8;
                    end
                 
                       
                    ZoomCord=[mpos(2)-height/8,mpos(1)-width/8,mpos(2)+height/8,mpos(1)+width/8]; 
                    ZoomCord=round(ZoomCord);
        
    end


%  ----- handling keyboard inputs -----

function KeyPressEvent(src,evt)
        ch = get(gcf,'CurrentCharacter');
        %disp(ch);
        switch ch
            
            case '.'
                if frame < frames
                    frame = frame+1;
                end
                set(slider1,'Value', frame);
                Update();
           
            case ','
                if frame >= 2
                    frame = frame-1;
                end
                set(slider1,'Value', frame);
                Update();
            case 'c'
                 map=markermap(:,:,frame);
                 map(map>0)=0;
                 markermap(:,:,frame)=map; 
                 Update();
            case 'p'
                if (ViewMode==1)
                     ViewMode=2;
                else
                     ViewMode=1;
                end
                  set(ViewModeMenu,'Value', ViewMode);
                   Update();      
            case 'e'
                  if (PaintMode == 1 && eraserSize==1)  
                  PaintMode=0; eraserSize=1;
                  else
                  PaintMode=1; eraserSize=1;   
                  end
                Update();
                
             case 'r'
                  
                  if (PaintMode == 1 && eraserSize==3)  
                  PaintMode=0; eraserSize=1;
                  else
                  PaintMode=1; eraserSize=3;
                  end
                Update();
                
             case 't'
                  if (PaintMode == 1 && eraserSize==5)  
                  PaintMode=0; eraserSize=1;
                  else
                 PaintMode=1; eraserSize=5;
                 end
                Update();
                
             case 'd'
                if (PaintMode==0 || PaintMode==1)
                    PaintMode=2;
                else
                    PaintMode=0;
                end
                Update();   
                 
            case 'z'
                if (ZoomMode==1)
                    
                    ZoomMode=2;
                    
                    ZoomCoordinatesRegion();
                 
                else
                    ZoomMode=1;
                end
                Update();
                
            case 'x'
                if (ZoomMode==1)
                    
                    ZoomMode=2;
                    
                    ZoomCoordinatesPoint();
                  
                else
                    ZoomMode=1;
                end
                Update();
                
                
            case '1'
                ViewMode=1;
                set(ViewImageMenu,'Value', 1);
                Update();
            case '2'
                ViewMode=2;
                set(ViewImageMenu,'Value', 2);
                Update();
            case '3'
                ViewMode=3;
                set(ViewImageMenu,'Value', 3);
                Update();
                
                
                
        end
end

end




   
   
