% Tracking GUI for cell tracking

function fig = TrackingCorrectionGUI(Istack)

% read projected image stack
%Istack=imread3(filename);

s=size(Istack);

cellImages=[];

% Create new figure        
fig = figure;

fig1=[];

% initial configuration
frame=1;

frames=s(3);

width=s(2); height=s(1);

EraseMode = 0; % 1=enable eraser
eraseSize=5;
DrawMode=0; % 1=enable drawing
mpos=[0,0]; % mouse position

taillength=5;

% viewing mode 
% 0=no, 1=single cells, 2=short track ends, 
MarkerMode=1;
trackID=1;

listofSingleCells=[];
listofShortTracks=[];


xpos=0; ypos=0;

% 1=original images, 2=skeletons, 3=RGB cell labels 4=overlay
ViewMode=1;
ratio=50;
marking=0;
drag=0;

skelthreshold=100;

ZoomMode=1; % normal
ZoomCord=[ 3*height/8,3*width/8,5*height/8,5*width/8];

% create random color map
cmap=ones(16,3);

cmap(:,1)=randperm(16)/16;
cmap(:,2)=0.7*ones(1,16);
cmap(:,3)=ones(1,16);



% cell markers and tracks data
markermap=zeros(s(1),s(2),frames,'uint8');
trackmap=zeros(s(1),s(2),frames,'uint16');
Skeletons=zeros(s(1),s(2),frames,'uint8');

% tracking data
tracklength=[];
Ptracks=[];
trackstarts=[];
trackstartX=[]; trackstartY=[]; 



% ------- Setting Up GUI -------------

set(fig,...
    'Name', 'Tracking Correction',... 
    'Color', [0.314 0.314 0.314],...
    'Units', 'normalized',...
    'MenuBar','none',...
    'Position', [0 0 0.8 0.8]);

    % create long slider to control image time frame
    slider1 = uicontrol( fig ...
        ,'style'    ,'slider'   ...
        ,'units'    ,'normalized' ...
        ,'position' ,[0.15 0.05 0.80 0.03] ...
        );

    set(slider1,'min', 1);
    set(slider1,'max', frames);
    set(slider1,'Value', frame);

    addlistener(slider1,'ContinuousValueChange',@sliderActionEventTime);
    
    FrameNumLabel = uicontrol(fig ...
        ,'style'    ,'text' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'position' ,[0.05 0.05 0.09 0.03] ...
        ,'string'   ,sprintf('Frame %d/%d',frame,frames) ...
        );
   
    
    slider4 = uicontrol( fig ...
        ,'style'    ,'slider'   ...
        ,'units'    ,'normalized' ...
        ,'position' ,[0.4 0.9 0.07 0.02] ...
        );

    set(slider4,'min', 0);
    set(slider4,'max', 100);
    set(slider4,'Value',50);
    addlistener(slider4,'ContinuousValueChange',@sliderActionEventRatio);
    
    
    set (gcf, 'WindowButtonMotionFcn', @MouseMoveEvent);
    
    set(fig,'KeyPressFcn',@KeyPressEvent);
    
    
    % ------  buttons --------------
    
    SuggestButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Find Cells'...
        ,'Position', [0.05 0.9 0.1 0.04]...
        ,'Callback', @SuggestMarkers ... 
        );
    
    
    TrackBackwardButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', '<< C'...
        ,'Position', [0.15 0.9 0.05 0.04]...
        ,'Callback', @TrackMarkersBackward ... 
        );
    
    TrackForwardButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'C >>'...
        ,'Position', [0.2 0.9 0.05 0.04]...
        ,'Callback', @TrackMarkersForward ... 
        );
    
    
    ClearButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Clear Frame'...
        ,'Position', [0.25 0.95 0.1 0.04]...
        ,'Callback', @ClearMarkers ... 
        );
    
    ClearAllButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Clear All'...
        ,'Position', [0.25 0.9 0.1 0.04]...
        ,'Callback', @ClearAllMarkers ... 
        );
    
   
    
    CreateButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Track and Analyse'...
        ,'Position', [0.6 0.9 0.1 0.04]...
        ,'Callback', @CreateTracks ... 
        );
    
    ExportSequenceButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Export Sequence'...
        ,'Position', [0.01 0.2 0.1 0.04]...
        ,'Callback', @ExportTracks ... 
        );
    
     PlayButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 12 ...
        ,'String', '>'...
        ,'Position', [0.01 0.05 0.03 0.03]...
        ,'Callback', @PlaySequence ... 
        );
    
   ViewIssuesMenu = uicontrol('Style', 'popup',...
           'String', {'None','Problems','Tracks <= 3','Tracks <= 6','Tracks <= 9','Tracks <= 12','Tracks <= 15','Tracks <= 18','Tracks <= 21'}...
           ,'units'    ,'normalized' ...
           ,'FontSize', 14 ...
           ,'Position', [0.15 0.94 0.1 0.05]...
           ,'Callback', @ViewIssuesMode...
           );       
       
    ViewImageMenu = uicontrol('Style', 'popup',...
           'String', {'Image Mode','Skeleton Mode', 'Cell Mode', 'Overlay Mode'}...
           ,'units'    ,'normalized' ...
           ,'FontSize', 14 ...
           ,'Position', [0.37 0.94 0.15 0.05]...
           ,'Callback', @SetImageMode...
           );    
    
     LoadSkeletonButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Load Skeleton'...
        ,'Position', [0.75 0.9 0.1 0.04]...
        ,'Callback', @LoadSkeletons ... 
        ); 
       
      
     LoadTracksButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Load Markers'...
        ,'Position', [0.75 0.95 0.1 0.04]...
        ,'Callback', @LoadTracks ... 
        );
    
    LoadRGBButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Load Cells'...
        ,'Position', [0.05 0.95 0.1 0.04]...
        ,'Callback', @LoadCellImages ... 
        );
    
    SaveTracksButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Save Markers'...
        ,'Position', [0.85 0.95 0.1 0.04]...
        ,'Callback', @SaveTracks ... 
        );
    
    ExportTracksButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Export Skeleton'...
        ,'Position', [0.85 0.9 0.1 0.04]...
        ,'Callback', @ExportSkeleton ... 
        );
    
    StatsButton = uicontrol(fig ...
        ,'Style', 'pushbutton' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'String', 'Show Stats'...
        ,'Position', [0.55 0.95 0.1 0.04]...
        ,'Callback', @ShowStats ... 
        );
  
   
    FeedbackLabel = uicontrol(fig ...
        ,'style'    ,'text' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'position' ,[0 0 0.15, 0.03] ...
        ,'string'   ,'Ready' ...
        );
    
    IssuesLabel = uicontrol(fig ...
        ,'style'    ,'text' ...
        ,'units'    ,'normalized' ...
        ,'FontSize', 14 ...
        ,'position' ,[0 0.8 0.1, 0.03] ...
        ,'string'   ,'No issues' ...
        );

Update();




function Update()
    
        set(FeedbackLabel,'string','Ready');
        
       
        figure(fig);
       
        set(slider1,'Value', frame);
        
        % ---------- Show Stack -----------------
        
        img=imshow(createOverlayImage,[]);
        
              
            
        % no zoom
        if (ZoomMode==1)
            
            
        for n=1:3,
        xp(1)=n*width/4; xp(2)=n*width/4;
        yp(1)=0;yp(2)=height;
        line(xp,yp,'LineStyle','--');
        
        yp(1)=n*height/4; yp(2)=n*height/4;
        xp(1)=0;xp(2)=width;
        line(xp,yp,'LineStyle','--');
        end
            
         % highlight issues with marker
        if (MarkerMode > 1 && trackID > 0 && DrawMode==0 && EraseMode==0)
        
            hold on,
            
            fig3=plot(xpos,ypos,'ro','MarkerSize', 30, 'LineWidth',4); 
            hold off
            
            set(fig3,'ButtonDownFcn',@MouseButtonDown);
            
        end    
            
        
        % unallocated markers
        [vy,vx]=find(markermap(:,:,frame)>=253);
        hold on
        fig1=plot(vx,vy,'r*','MarkerSize', 5); 
        hold off
        
         % tracked markers
        [vy,vx]=find(trackmap(:,:,frame)>0);
        hold on
        fig2=plot(vx,vy,'g*','MarkerSize', 5); 
        hold off
        
        
        if (EraseMode == 1)
        
            hold on,
            
            fig4=plot(mpos(1),mpos(2),'ro','MarkerSize', 3*eraseSize); 
            hold off
            
            %Skeletons(round(mpos(2)),round(mpos(1)),frame)=0;
            
            
            set(fig4,'ButtonDownFcn',@MouseButtonDown);
            
        end
        
        if (DrawMode == 1)
        
            hold on,
            
            fig4=plot(mpos(1),mpos(2),'yo','MarkerSize', 5); 
            hold off
            
            %Skeletons(round(mpos(2)),round(mpos(1)),frame)=255;
            set(fig4,'ButtonDownFcn',@MouseButtonDown);
            
        end
        
        
        
        
       
        % -------------------zoom mode --------------------------------------
        else
              
            
         % highlight issues with marker
        if (MarkerMode > 1 && trackID > 0 && DrawMode==0 && EraseMode==0)
        
            hold on,
            
            fig3=plot(xpos-ZoomCord(2),ypos-ZoomCord(1),'ro','MarkerSize', 80, 'LineWidth',7); 
            %fig3=plot(100,200,'ro','MarkerSize', 50);
            hold off
            
            set(fig3,'ButtonDownFcn',@MouseButtonDown);
            
        end    
            
        [vy,vx]=find(markermap(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame)>=253);
        hold on
        fig1=plot(vx,vy,'r*','MarkerSize', 5); 
        hold off 
        
        [vy,vx]=find(trackmap(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame)>0);
        hold on
        fig2=plot(vx,vy,'g*','MarkerSize', 5); 
        hold off
         
        
        if (EraseMode == 1)
        
            hold on,
            
            fig4=plot(mpos(1),mpos(2),'ro','MarkerSize', 3*eraseSize); 
            hold off
            
            %Skeletons(round(mpos(2)+ZoomCord(1)),round(mpos(1)+ZoomCord(2)),frame)=0;
            set(fig4,'ButtonDownFcn',@MouseButtonDown);
            
        end
        
        if (DrawMode == 1)
        
            hold on,
            
            fig4=plot(mpos(1),mpos(2),'yo','MarkerSize', 5); 
            hold off
            
            %Skeletons(round(mpos(2)+ZoomCord(1)),round(mpos(1)+ZoomCord(2)),frame)=255;
            set(fig4,'ButtonDownFcn',@MouseButtonDown);
            
        end
        
        
        end
        
        set(fig1,'ButtonDownFcn',@MouseButtonDown);
        set(fig2,'ButtonDownFcn',@MouseButtonDown);
        set(img,'ButtonDownFcn',@MouseButtonDown);
        
         set(gcf, 'WindowButtonUpFcn',@MouseButtonUp);
     
        set(FrameNumLabel,'string',sprintf('Frame %d/%d',frame,frames));
        
       
        
end

% --- Event Handlers -----------------------


function sliderActionEventTime(src,evt)
        frame = round(get(src,'Value'));        
        Update();     
end


function sliderActionEventRatio(src,evt)
        ratio = round(get(src,'Value'))/100;
        %set(RatioLabel,'string',sprintf('Ratio %1.2f',ratio));
        Update();
end


%  Find markers from the skeleton image
function SuggestMarkers(src,evt)
        
    set(FeedbackLabel,'string',' Finding cells .....');
    drawnow;
    
     % ---- finding markers in ONE frame  -------------
    
    if (ViewMode==1)
        centroids = FindCellsFunc(Istack(:,:,frame),threshold);
        
        for n=1:length(centroids)
        
          if(centroids(n,1) > 3 && centroids(n,2) > 3 && centroids(n,1) < width-3 && centroids(n,2) < height-3)
          markermap(centroids(n,2),centroids(n,1),frame)=255;
         end
        end
         
    else
        
        % ---- finding markers in all frames  -------------
        
        if (frame==1)
            startframe=1;
            stopframe=frames;
        else
           startframe=frame;
            stopframe=frame; 
        end
        
        
        for t=startframe:stopframe,
            
        set(FeedbackLabel,'string',' Finding cells .....');
        drawnow;
            
        frame=t;
         centroids = FindCellsFromSkeletonFunc(ones(size(Skeletons(:,:,frame))),Skeletons(:,:,frame),skelthreshold);
         
         for n=1:length(centroids)
        
         % removing border positions    
             
         if(centroids(n,1) > 3 && centroids(n,2) > 3 && centroids(n,1) < width-3 && centroids(n,2) < height-3)
          
          markermap(centroids(n,2),centroids(n,1),frame)=255;
         end
         end
         
         Update();
         drawnow;
         
        
        end
         
     
    end
    
    set(FeedbackLabel,'string','Ready');
    Update();
        
end


% selection next problem area
function TrackMarkersBackward(src,evt)
        
    trackID=trackID-1;
    EraseMode=0; DrawMode=0;
     
   % find problems in tracking array
   if (MarkerMode==2 && trackID > 0)
    
       frame=listofSingleCells(trackID,1);
       xpos=listofSingleCells(trackID,2); 
       ypos=listofSingleCells(trackID,3); 
       
       if (ZoomMode==2)
         mpos(1)=xpos;
         mpos(2)=ypos; 
         ZoomCoordinates();
       end
       
      
       set(IssuesLabel,'string',sprintf('Issue %d of %d',trackID,length(listofSingleCells)));
       
        % store issue number in file 
       save('IssueNumber.txt','trackID','-ASCII');
   end
   
   % find short tracks
   if (MarkerMode==3 && trackID > 0)
    
       frame=listofShortTracks(trackID,1);
       xpos=listofShortTracks(trackID,2); 
       ypos=listofShortTracks(trackID,3);
       
       if (ZoomMode==2)
         mpos(1)=xpos;
         mpos(2)=ypos; 
         ZoomCoordinates();
       end
       
       set(IssuesLabel,'string',sprintf('Issue %d of %d',trackID,length(listofShortTracks)));
       
       % store issue number in file 
       save('IssueNumber.txt','trackID','-ASCII');
   end
   
    if (MarkerMode==1)
       set(IssuesLabel,'string','No issues');   
    end
   
   
    Update();
        
end


% select next probelm area
function TrackMarkersForward(src,evt)
        
  
   trackID=trackID+1;
   EraseMode=0; DrawMode=0;  
   
   
   % find problems in tracking array
   if (MarkerMode==2 && trackID < length(listofSingleCells))
    
       frame=listofSingleCells(trackID,1);
       xpos=listofSingleCells(trackID,2); 
       ypos=listofSingleCells(trackID,3); 
       
       if (ZoomMode==2)
         mpos(1)=xpos;
         mpos(2)=ypos; 
         ZoomCoordinates();
       end
       
       set(IssuesLabel,'string',sprintf('Issue %d of %d',trackID,length(listofSingleCells)));
       
        % store issue number in file 
       save('IssueNumber.txt','trackID','-ASCII');
       
   end
   
   
   % find short tracks
   if (MarkerMode > 2 && trackID < length(listofShortTracks))
    
       frame=listofShortTracks(trackID,1);
       xpos=listofShortTracks(trackID,2); 
       ypos=listofShortTracks(trackID,3); 
       
       if (ZoomMode==2)
         mpos(1)=xpos;
         mpos(2)=ypos; 
         ZoomCoordinatesPoint();
       end
       
      set(IssuesLabel,'string',sprintf('Issue %d of %d',trackID,length(listofShortTracks)));
      
       % store issue number in file 
       save('IssueNumber.txt','trackID','-ASCII');
       
   end
   
   if (MarkerMode==1)
       set(IssuesLabel,'string','No issues');   
    end
    
    Update();
        
end


function CreateTracks(src,evt)
        
    set(FeedbackLabel,'string',' Creating tracks .....');
    drawnow;
    
    %[trackmap, tracklength]=cellTracking(markermap, 20);
    [trackmap, Ptracks, tracklength, trackstarts, trackstartX, trackstartY ]=cellTracking4( markermap,25);
    
    set(FeedbackLabel,'string','Ready');
     
    %IdentifyProblems();
    
    Update();
        
end


% identify issues with the cell tracks
function IdentifyProblems()
    
    listofSingleCells=[];
    listofShortTracks=[];
    
    trackID=1;
    
    for f=1:s(3)-1,
        
    [vy,vx]=find(markermap(:,:,f) == 255);
    
    % assign single cells to list
    for n=1:length(vx),
    
     if (trackmap(vy(n),vx(n)) == 0)
         
     listofSingleCells(trackID,1)=f;
     listofSingleCells(trackID,2)=vx(n);
     listofSingleCells(trackID,3)=vy(n);
     
     trackID=trackID+1;
     
     end
    
    end
   
    end
    
    trackID=1;
    
    disp(taillength);
    
    for n=1:length(tracklength),
      
        if (tracklength(n) <= taillength)
            
            % jump to track endings of short tracks
           sc=trackstarts(n);
           xp=Ptracks(n,sc+tracklength(n),1);
           yp=Ptracks(n,sc+tracklength(n),2);
           
           if (xp < 50 || xp > width-50) continue;
           end
           
           if (yp < 50 || yp > height-50) continue;
           end
           
           listofShortTracks(trackID,1)=sc+tracklength(n);
           listofShortTracks(trackID,2)=Ptracks(n,sc+tracklength(n),1);
           listofShortTracks(trackID,3)=Ptracks(n,sc+tracklength(n),2);
           listofShortTracks(trackID,4)=n; % track id
           
           trackID=trackID+1;
              
        end
          
    end
    
    
    trackID=0;
    
           
end


function ExportTracks(src,evt)
        
        
    %ExportTrackLabels(trackmap,Skeletons);
    
    
      
        
end



function PlaySequence(src,evt)
    
    if (frame < frames)
       frame=frame+1;
       Update();
        
    end
        
end

function ExportSkeleton(src,evt)

    f=1;
    
    filename=uiputfile('*.tif','Export tracked cells');
    if (filename==0)
     disp('no file selected');
    else
        
       imwrite3(Skeletons,filename);
       disp(filename);
    end     
         
end


function ClearMarkers(src,evt)
        
  % erase all markers in frame
  markermap(:,:,frame) = zeros(s(1),s(2),'uint8');
  trackmap(:,:,frame) = zeros(s(1),s(2),'uint16');
  Update();
        
end


function ClearAllMarkers(src,evt)
        
  % erase all markers in stack
  markermap=zeros(s(1),s(2),frames,'uint8');
  trackmap=zeros(s(1),s(2),frames,'uint16');
  Update();
        
end





function SetImageMode(src,evt)
        
        ViewMode=get(src,'Value');
        Update();
        
end

function ShowStats(src,evt)
        
      figure, hist(tracklength(:),frames);
        
end

function ViewIssuesMode(src,evt)
        
        MarkerMode=get(src,'Value');
        trackID=1;
        if (MarkerMode==1) 
             set(IssuesLabel,'string','No issues'); 
        elseif (MarkerMode==2)
             IdentifyProblems();
             set(IssuesLabel,'string',sprintf('Issue %d of %d',trackID,length(listofSingleCells)));
        elseif(MarkerMode > 2)
            taillength=3*(MarkerMode-2);
            IdentifyProblems();
            set(IssuesLabel,'string',sprintf('Issue %d of %d',trackID,length(listofShortTracks)));
             
        end
        
       
        
        Update();
        
end


function LoadTracks(src,evt)
        
    [filename,path]=uigetfile('*.tif','Load Markers');
    if (filename==0)
     disp('no file selected');
    else
        
      set(FeedbackLabel,'string','Loading .....');
      drawnow;
      markermap=imread3(strcat(path,filename));
      disp(filename);
    end
    Update();
    
end

function SaveTracks(src,evt)
        
   [filename,path]=uiputfile('*.tif','Save Markers');
    if (filename==0)
     disp('no file selected');
    else
      imwrite3(uint8(markermap),strcat(path,filename));
       disp(filename);
    end 
   
end

function LoadSkeletons(src,evt)
        
    [filename,path]=uigetfile('*.tif','Load Skeletons');
    if (filename==0)
     disp('no file selected');
    else
      set(FeedbackLabel,'string','Loading .....');
      drawnow;
      Skeletons=imread3(strcat(path,filename));
      disp(filename);  
      skelthreshold=mean(Skeletons(:));
    end
    ViewMode=2;
    set(ViewImageMenu,'Value', 2);
    Update();  
end

function LoadCellImages(src,evt)
        
    [filename,path]=uigetfile('*.tif','Load Skeletons');
    if (filename==0)
     disp('no file selected');
    else
      set(FeedbackLabel,'string','Loading .....');
      drawnow;
      cellImages=imread3(strcat(path,filename));
      disp(filename);
      ViewMode=3;
    end
    
    set(ViewImageMenu,'Value', 3);
    Update();  
end



function img=createOverlayImage()
    
 img=[];
 % rgb overlay
 if (ViewMode==4)
 
    if (ZoomMode > 1)      
        I1=Skeletons(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);
        I2=Istack(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);
    else
        I1=Skeletons(:,:,frame); 
        I2=Istack(:,:,frame);
    end 
     
    img1(:,:,1)=zeros(size(I1));
    img1(:,:,2)=2*double(I2);
    img1(:,:,3)=zeros(size(I1));
    
    img2(:,:,1)=double(I1);
    img2(:,:,2)=double(I1);
    img2(:,:,3)=double(I1);
    
      
    img=uint8(ratio*double(img1)+((1-ratio)*double(img2)));
     
 
 % cell images
 elseif (ViewMode==3)
    if (ZoomMode > 1)      
        rgb2=cellImages(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),:,frame);
    else 
        rgb2=cellImages(:,:,:,frame);
    end 
    
    img=rgb2;
    
 else
    % no overlay
     if (ZoomMode > 1)
            if (ViewMode == 2)
             img=Skeletons(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);   
            else
             img=Istack(ZoomCord(1):ZoomCord(3),ZoomCord(2):ZoomCord(4),frame);
            end
        else
          if (ViewMode == 2)
           img=Skeletons(:,:,frame);  
          else
           img=Istack(:,:,frame);
          end
        
     end
        
 end
     
end



% ---- handling mouse buttons ----

function MouseButtonDown(src,evt)
    
       pt = get(gca,'Currentpoint');
       
       mouseclick  = get(gcf,'SelectionType');
       disp(mouseclick);
       
       if strcmp(mouseclick ,'normal')
       
       crd = round([pt(1,1), pt(1,2)]);
       
       
       if (ZoomMode ==1)
           crd = round([pt(1,1), pt(1,2)]);
       else
           crd = round([pt(1,1)+ZoomCord(2), pt(1,2)+ZoomCord(1)]);
       end
       
       
        if (EraseMode==1 || DrawMode==1) 
           drag=1;
           marking=0;
           return;  
        end
       
       
       
       % check bounds
       if (crd(1) < 1) crd(1)=1; end
       if (crd(1) > s(2)) crd(1)=s(2); end
       if (crd(2) < 1) crd(2)=1; end
       if (crd(2) > s(1)) crd(2)=s(1); end
       
      
       markermap(crd(2), crd(1), frame)=255;
      
       Update();
       end
       
       if strcmp(mouseclick ,'alt')
           
          crd = round([pt(1,1), pt(1,2)]);
          
          if (ZoomMode ==1)
           crd = round([pt(1,1), pt(1,2)]);
          else
           crd = round([pt(1,1)+ZoomCord(2), pt(1,2)+ZoomCord(1)]);
          end
          
          removeMarker(crd(2), crd(1), 5);
          Update();
           
       end
        
       
        
end


function MouseButtonUp(src,evt)
    
      if (EraseMode==1 || DrawMode==1) 
           drag=0;
           marking=0;
           return;  
      end
end

function MouseMoveEvent (object, eventdata)
    
     pt = get (gca, 'CurrentPoint');
     mpos=([pt(1,1) pt(1,2)]);
    
     
    if (EraseMode==1 || DrawMode==1)
        
        disp(drag);
        % deleting skeleton
        if (EraseMode==1 && drag==1)
           removeSkeleton(round(mpos(1)), round(mpos(2)), round(eraseSize/2));
           marking=marking+1;
        end
        
        % drawing skeleton
        if (DrawMode==1 && drag==1)
           paintSkeleton(round(mpos(1)), round(mpos(2)), 2); 
           marking=marking+1;
        end
        
        % dealing with the updating after drawing every so often
        if (marking > 9)
        Update();  
        marking=0; 
        end
        
        if (drag==0) 
            Update();
        end
    end 
    

end

function removeMarker(x,y, mindistance)
    
     
     [vx,vy]=find(markermap(:,:,frame)>0);
    
     for n=1:length(vx)
         if (hypot(vx(n)-x,vy(n)-y) < mindistance)
           
            markermap(vx(n),vy(n),frame)=0; 
             
         end    
     end
     
     
     [vx,vy]=find(trackmap(:,:,frame)>0);
    
     for n=1:length(vx)
         if (hypot(vx(n)-x,vy(n)-y) < mindistance)
           
            trackmap(vx(n),vy(n),frame)=0; 
             
         end    
     end
     
end

function paintSkeleton(x,y,size)

    
    if (ZoomMode==2)
       x=x+ZoomCord(2);
       y=y+ZoomCord(1);
    end
   
     if (x < size) x=size; end
     if (x > s(2)-size) x=s(2)-size; end
     if (y < size) y=size; end
     if (y > s(1)-size) y=s(1)-size; end


   Skeletons(y,x,frame)=150;
	%Skeletons(y-1,x,frame)=150;
	%Skeletons(y+1,x,frame)=150;
	%Skeletons(y,x-1,frame)=150;
	%Skeletons(y,x+1,frame)=150;
	%Skeletons(y-1,x-1,frame)=100;
	%Skeletons(y+1,x+1,frame)=100;
	%Skeletons(y+1,x-1,frame)=100;
    %Skeletons(y-1,x+1,frame)=100;
     
   
end



function removeSkeleton(x,y,size)

    
    if (ZoomMode ==2)
       x=x+ZoomCord(2);
       y=y+ZoomCord(1);
    end
   
     if (x < size) x=size; end
     if (x > s(2)-size) x=s(2)-size; end
     if (y < size) y=size; end
     if (y > s(1)-size) y=s(1)-size; end

     Skeletons(y-size:y+size,x-size:x+size,frame)=0;
   
end




%  ----- handling keyboard inputs -----

function KeyPressEvent(src,evt)
        ch = get(gcf,'CurrentCharacter');
       
        switch ch
            
            case 's'
                if frame < frames
                    frame = frame+1;
                end
                set(slider1,'Value', frame);
                Update();
                
            case 'a'
                if frame >= 2
                    frame = frame-1;
                end
                set(slider1,'Value', frame);
                Update();
            case 'c'  % clear markers
                 map=markermap(:,:,frame);
                 map(map>0)=0;
                 markermap(:,:,frame)=map; 
                 Update();
            case 'i' % load last issue number
                trackID=load('IssueNumber.txt')-1;
                TrackMarkersForward(10,10);
                
            case 'e'
                if (EraseMode==0)
                    EraseMode=1;
                    DrawMode=0;
                    eraseSize=1.5;
                    else
                  EraseMode=0; 
                end
                Update();
                
             case 'd'
                if (DrawMode==0)
                    EraseMode=0;
                    DrawMode=1;
                else
                    DrawMode=0;
                end
                Update();
            case 'z'
                if (ZoomMode==1)
                    
                    ZoomMode=2;
                  
                    ZoomCoordinates();
                else
                    ZoomMode=1;
                end
                Update();
                
                
            case '1'
                ViewMode=1;
                set(ViewImageMenu,'Value', 1);
                Update();
            case '2'
                ViewMode=2;
                set(ViewImageMenu,'Value', 2);
                Update();
            case '3'
                ViewMode=4;
                set(ViewImageMenu,'Value', 4);
                Update();
            case '4'
                ViewMode=3;
                set(ViewImageMenu,'Value', 3);
                Update();
                
            case ','
                TrackMarkersBackward(src,evt);
            case '.'
                TrackMarkersForward(src,evt);
                
        end
end


    function ZoomCoordinates()
        
        
        
                    if (mpos(1) <= width/8)
                        mpos(1)=width/8+1;
                    end
                    if (mpos(2) <= height/8)
                        mpos(2)=height/8+1;
                    end
                    
                    if (mpos(1) >= 7*width/8)
                        mpos(1)=7*width/8;
                    end
                    if (mpos(2) >= 7*height/8)
                        mpos(2)=7*height/8;
                    end
                        
                    
%                     if (mpos(1) < width/4)
%                         mpos(1)=width/8+1; 
%                     elseif(mpos(1) < width/2)
%                          mpos(1)=3*width/8+1;   
%                     elseif(mpos(1) < 3*width/4)
%                          mpos(1)=5*width/8+1;
%                     else
%                         mpos(1)=7*width/8;
%                     end
%                     
%                     
%                     if (mpos(2) < height/4)
%                         mpos(2)=height/8+1; 
%                     elseif(mpos(2) < height/2)
%                          mpos(2)=3*height/8+1;   
%                     elseif(mpos(2) < 3*height/4)
%                          mpos(2)=5*height/8+1;
%                     else
%                         mpos(2)=7*height/8;
%                     end
                 
                       
                    ZoomCord=[mpos(2)-height/8,mpos(1)-width/8,mpos(2)+height/8,mpos(1)+width/8]; 
                    ZoomCord=round(ZoomCord);
        
    end


function ZoomCoordinatesPoint()
        
                    if (mpos(1) <= width/8)
                        mpos(1)=width/8+1;
                    end
                    if (mpos(2) <= height/8)
                        mpos(2)=height/8+1;
                    end
                    
                    if (mpos(1) >= 7*width/8)
                        mpos(1)=7*width/8;
                    end
                    if (mpos(2) >= 7*height/8)
                        mpos(2)=7*height/8;
                    end
                    
                    ZoomCord=[mpos(2)-height/8,mpos(1)-width/8,mpos(2)+height/8,mpos(1)+width/8]; 
                    ZoomCord=round(ZoomCord);
end
                   

end




   
   
