#include "mex.h"
#include "utilah.h"


// ---- calculating Ex -----
void calcEx( matx* I1, matx* I2, matx* Ex)
{
  int x,y;

  for(x=1;x < I1->cols-1; x++)
  {
    for(y=1;y < I1->rows-1; y++)
	{
          Ex->rptr[x][y] = (I1->rptr[x+1][y] + I1->rptr[x+1][y+1] +
	    	    I2->rptr[x+1][y] + I2->rptr[x+1][y+1])/4.0
		  -(I1->rptr[x][y] + I1->rptr[x][y+1] +
		    I2->rptr[x][y] + I2->rptr[x][y+1])/4.0;
	}
     }
}


// ---- calculating Ey -----
void calcEy( matx* I1, matx* I2, matx* Ey)
{

  int x,y;

  for(x=1;x < I1->cols-1; x++)
  {
    for(y=1;y < I1->rows-1; y++)
	{

	Ey->rptr[x][y] = (I1->rptr[x][y+1] + I1->rptr[x+1][y+1] +
	    	    I2->rptr[x][y+1] + I2->rptr[x+1][y+1])/4.0
		  -(I1->rptr[x][y] + I1->rptr[x+1][y] +
		    I2->rptr[x][y] + I2->rptr[x+1][y])/4.0;
	}
  }
}

// ---- calculating Et -----
void calcEt( matx* I1, matx* I2, matx* Et)
{

  int x,y;

  for(x=1;x < I1->cols-1; x++)
  {
    for(y=1;y < I1->rows-1; y++)
	{
	  Et->rptr[x][y] = (I2->rptr[x][y] + I2->rptr[x+1][y] +
	    	    I2->rptr[x][y+1] + I2->rptr[x+1][y+1])/4.0
		  -(I1->rptr[x][y] + I1->rptr[x+1][y] +
		    I1->rptr[x][y+1] + I1->rptr[x+1][y+1])/4.0;
	}
  }
}

// ---- average velocities returns average matrix as I2-----
void vels_avg(matx* I1, matx* I2)
{
 int x,y;

 for(x=1;x < I1->cols-1; x++)
 {
    for(y=1;y < I1->rows-1; y++)
       {

        I2->rptr[x][y] = (I1->rptr[x-1][y] + I1->rptr[x][y+1]+
			I1->rptr[x+1][y] + I1->rptr[x][y-1])/6.0 +
       		       (I1->rptr[x-1][y-1] + I1->rptr[x-1][y+1]+
		        I1->rptr[x+1][y+1]+I1->rptr[x+1][y-1])/12.0;
	}
 }

 // corner points/lines
 for (x=0; x < I1->cols; x++)
 {
   I2->rptr[x][0]=I2->rptr[x][1];
   I2->rptr[x][I2->rows-1]=I2->rptr[x][I2->rows-2];
 }
 for (y=0; y < I1->rows; y++)
 {
   I2->rptr[0][y]=I2->rptr[1][y];
   I2->rptr[I1->cols-1][y]=I2->rptr[I1->cols-2][y];
 }
  I2->rptr[0][0]=I2->rptr[1][1];
  I2->rptr[0][I1->rows-1]=I2->rptr[1][I1->rows-2];
  I2->rptr[I1->cols-1][0]=I2->rptr[I1->cols-2][1];
  I2->rptr[I1->cols-1][I1->rows-1]=I2->rptr[I1->cols-2][I1->rows-2];
}

void calc_vels(matx* Vx1, matx* Vy1, matx* Vx2, matx* Vy2, matx* Ex, matx* Ey, matx* Et, double alpha)
{
  int x,y;
  matx* Ave0 = mx_new(Vx1->rows, Vx1->cols);
  matx* Ave1 = mx_new(Vx1->rows, Vx1->cols);

  vels_avg(Vx2, Ave0);
  vels_avg(Vy2, Ave1);

  for(x=0; x < Vx1->cols; x++)
  {
    for(y=0; y < Vx1->rows; y++)
    {
	  Vx1->rptr[x][y] = Ave0->rptr[x][y]-Ex->rptr[x][y]*
		(Ex->rptr[x][y]*Ave0->rptr[x][y]+Ey->rptr[x][y]*Ave1->rptr[x][y]+Et->rptr[x][y])
		/(alpha*alpha+Ex->rptr[x][y]*Ex->rptr[x][y]+Ey->rptr[x][y]*Ey->rptr[x][y]);

	  Vy1->rptr[x][y] = Ave1->rptr[x][y]-Ey->rptr[x][y]*
		(Ex->rptr[x][y]*Ave0->rptr[x][y]+Ey->rptr[x][y]*Ave1->rptr[x][y]+Et->rptr[x][y])
		/(alpha*alpha+Ex->rptr[x][y]*Ex->rptr[x][y]+Ey->rptr[x][y]*Ey->rptr[x][y]);

     }
    }

 mx_free( Ave0); mx_free( Ave1);

}


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

  matx *Ex, *Ey, *Et, *Vx1, *Vy1, *Ax, *By, *Vx2, *Vy2;
  int i;
  int no_rows;
  int no_cols;
  int steps;
  double alpha;

 // --- testing input and output values ---
 if (nrhs != 4) { mexErrMsgTxt(" usage: [vx, vy]=flow(I1,I2,alpha,iterations) "); }
 if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]))
     mexErrMsgTxt("Image input must be a double real matrix");
 if (!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]))
     mexErrMsgTxt("Image input must be a double real matrix");

  // -- initialising arrays and stuff ---
  no_rows = mxGetM(prhs[0]);
  no_cols = mxGetN(prhs[0]);
  alpha = mxGetScalar( prhs[2]);
  steps = (int)(mxGetScalar( prhs[3])+0.5);


  // --- input matrix ---
  Ax=mx_from_vector( mxGetPr(prhs[0]), no_rows, no_cols);
  By=mx_from_vector( mxGetPr(prhs[1]), no_rows, no_cols);
  
  
  // --- return matrix ---
  plhs[0] = mxCreateDoubleMatrix(no_rows,no_cols, mxREAL);
  Vx1=mx_from_vector( mxGetPr(plhs[0]), no_rows, no_cols);
  
  plhs[1] = mxCreateDoubleMatrix(no_rows,no_cols, mxREAL);
  Vy1=mx_from_vector( mxGetPr(plhs[1]), no_rows, no_cols);
  
  mx_clear( Vx1);
  mx_clear( Vy1);
  
  

  Ex = mx_new(no_rows, no_cols);
  Ey = mx_new(no_rows, no_cols);
  Et = mx_new(no_rows, no_cols);
  Vx2 = mx_new(no_rows, no_cols); mx_clear( Vx2);
  Vy2 = mx_new(no_rows, no_cols); mx_clear( Vy2);

  // --- calculating flow velocities -----
  calcEx( Ax, By, Ex);
  calcEy( Ax, By, Ey);
  calcEt( Ax, By, Et);


  for (i=0; i < steps; i++)
  {
    calc_vels( Vx1, Vy1, Vx2, Vy2, Ex, Ey, Et, alpha);
    calc_vels( Vx2, Vy2, Vx1, Vy1, Ex, Ey, Et, alpha);

  }

 mx_free( Ex); mx_free( Ey); mx_free( Et); mx_free( Vx1); mx_free( Vy1);
 mx_free( Ax); mx_free( By); mx_free( Vx2); mx_free( Vy2);

}

